/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP
 *
 * Copyright (C) 2004,2005 CNAM/INT - 2006 CNAM/INT/Filao
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Chanty Lim
 */
package org.mega.moods.generator;

import java.util.StringTokenizer;


/**
 * MooDS Generator v2.0
 * @author Romain PELLERIN
 */
public class LanguageTranslator {

	
	static class Language{
	
		private final String name;
		
		private Language(String name){
			this.name = name;
		}
		
		public String toString(){
			return name;
		}
		
		static final Language JAVA   = new Language("JAVA");
		static final Language FLASH  = new Language("FLASH");
		static final Language CPP    = new Language("CPP");
		static final Language CSHARP = new Language("CSHARP");
	}
    
    // t: tableau de strings a ecrire
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
    // t[3]: attributes
    // t[4]: constructeur
    // t[5]: setters/getters
    // t[6]: end of class
    
	public static String writeComments(Language language, String comments){
    	String code = "";
    	if (language.equals(Language.JAVA) || language.equals(Language.FLASH) || language.equals(Language.CPP) || language.equals(Language.CSHARP)) 
    		code = "// "+comments;
       	return code;
    }
    
    
    public static String writePackage(Language language, String packageName){
    	String code = "";
    	if (language.equals(Language.JAVA) ) code = "package "+ packageName + ";";
    	if (language.equals(Language.CSHARP) ) code = "namespace "+ packageName + "{";
    	return code;
    }
    
    
    public static String writeImport(Language language, String importName){
    	String code = "";
    	if (language.equals(Language.JAVA) ) code = "import "+ importName + ";";
    	
    	if (language.equals(Language.CPP) ) {
    		code = "#include ";
    		code += Util.isAnHeader(importName) ? "\""+importName+"\"" : "<"+importName+">";
    	}
    	if (language.equals(Language.CSHARP) ) code = "using "+ importName + ";";
    	return code;
    }
    
    private static class Util {

    	/**
    	 * Test if the file is an header file
    	 * 
    	 * @param name
    	 * @return true if classname extension is ".h"
    	 */
    	private static boolean isAnHeader (String filename){
    		String res ="";
    		StringTokenizer st = new StringTokenizer(filename,".");
    		while(st.hasMoreTokens()){
    			res = st.nextToken();
    		}
    		return res.equals("h") ? true : false;
    	}
    	
    	private static String lastToken (String filename){
    		String res ="";
    		StringTokenizer st = new StringTokenizer(filename,"/");
    		while(st.hasMoreTokens()){
    			res = st.nextToken();
    		}
    		return res;
    	}
    }
    /**
     * This method delete double includes when needed. It's only use for C++
     * @param include the string containing all includes
     * @return a new String without doublons
     */
    public static String deleteDoubleIncludes(String includes){
    	String[] st = includes.split("#include");
		// Begins at 1, cause 0 is the doc and 1 is "Object.h"
		for(int k = 2; k < st.length ; k++){
			//from the next one till the end
			for(int n = k+1; n < st.length; n++){
				// tests strings are the same, independently from layout
				if((st[k].compareTo((st[n])) == 0) || (st[k].compareTo((st[n])) == 1) || (st[k].compareTo((st[n])) == -1) && !st[n].equals("")){
					st[n] = "";
				}
			}
		}
		
		int caseToErase = 0, offset = 0;
		// counts the number of empty cases
		for(int k = 0; k < st.length; k++){
			if(st[k].equals("")){
				caseToErase++;
			}
		}
		
		// creates an array
		String[] res = new String[st.length-caseToErase];
		// copy non empty cases
		for(int k = 0; k < st.length; k++){
			if(!st[k].equals("")){
				res[offset] = st[k];
				offset++;
			}
		}	
		// replaces strings[0]
		String delete = res[0]; 
		for(int k = 1; k < res.length; k++){
			delete += "#include";
			delete += res[k];
		}
		return delete;
    }
    
  
    /**
     * For C++ generation : 
     * 		#ifndef __fileName_h__
     * 		#define __fileName_h__
     * @param 	fileName the name of the class
     * @return	the correspondin macro
     */
    public static String writeMacro(String fileName){
    	return "#ifndef __"+fileName+"_h__\n"+"#define __"+fileName+"_h__";
    }
    
    /**
     * For C++ generation : 
     * 		#endif
     * 
     * NB : New line after #endif to avoid warning or error with some compiler
     * @param 	fileName the name of the class
     * @return	the correspondin macro
     */
    public static String writeEndOfFile(){
    	return "#endif\n";
    }

    
    
  //For C++, let's consider that isImplement is unused.
    public static String writeClassDeclaration(Language language, String className, boolean isImplement, boolean isExtends, String _implements){
    	return writeClassDeclaration(language, className, isImplement, isExtends, _implements, (isExtends==true)? _implements:null);
    }
    
    
    //For C++, let's consider that isImplement is unused.
    public static String writeClassDeclaration(Language language, String className, boolean isImplement, boolean isExtends, String _implements, String _extends){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.CSHARP) ) {
    		code = "public class "+ className;
    		if (isExtends) code+= " extends " + _extends;
    		if (isImplement) code+= " implements " + _implements;
    		code+= " {";
    	}
    	if (language.equals(Language.CPP) ){
    		code = "class "+className;
    		if(isExtends) code += ": public "+_implements;
    	}
    	if (language.equals(Language.FLASH) ) code = "class "+ className + " {";
    	return code;
    }
    
    public static String writeConstructorDeclaration(Language language, String constructorName, String[][] params){
    	return writeConstructorDeclaration(language, constructorName, params, false, null);
    }
    
    public static String writeConstructorDeclaration(Language language,
            String constructorName, String[][] params,
            boolean isThrownException, String[] exceptions) {
        String code = "";
        if (language.equals(Language.JAVA) ){
            code = "public " + constructorName + "(";
            if (params != null && params.length > 0)
                for (int i = 0; i < params.length; i++) {
                	code += getJavaType(params[i][0]) + " " + params[i][1];
                    if ((i + 1) != params.length)
                        code += ",";
                }
            code += ")";
            if (isThrownException) {
                code += " throws ";
                for (int i = 0; i < exceptions.length; i++) {
                    code += exceptions[i];
                    if (i + 1 < exceptions.length)
                        code += ", ";
                }
            }
            code += " {";
        }
        if (language.equals(Language.CPP) ) {
            code = constructorName+ "(";
            if (params != null && params.length > 0)
                for (int i = 0; i < params.length; i++) {
                    code += getCppType(params[i][0]) + " " + params[i][1];
                    if ((i + 1) != params.length)
                        code += ",";
                }
            code += ")"+"\n\t{";
            
        }
        if (language.equals(Language.FLASH) ) {
            code = "public function " + constructorName + "(";
            if (params != null && params.length > 0)
                for (int i = 0; i < params.length; i++) {
                    code += params[i][1] + ":" + getFlashType(params[i][0]);
                    if ((i + 1) != params.length)
                        code += ",";
                }
            code += ") {";
        }
        if (language.equals(Language.CSHARP) ) {
            code = "public " + constructorName + "(";
            if (params != null && params.length > 0)
                for (int i = 0; i < params.length; i++) {
                	code += getCsharpType(params[i][0]) + " " + params[i][1];
                    if ((i + 1) != params.length)
                        code += ",";
                }
            code += ")";
            /*TODO: replace by a try/catch if (isThrownException) {
                code += " throws ";
                for (int i = 0; i < exceptions.length; i++) {
                    code += exceptions[i];
                    if (i + 1 < exceptions.length)
                        code += ", ";
                }
            }*/
            code += " {";
        }
        return code;
    }
    
    public static String writeDestructorDeclaration(String className){
    	return "~"+className+"()";
    }
    
    
    // En C++, on impose au variable d'etre private
    // De plus, seules les variables globales constantes peuvent etre initialisees
    public static String addGlobalVar(Language language, 
    									boolean isPublic,
    									boolean isFinal, 
    									boolean isStatic,
    									boolean isToInit,
    									String initValue,
    									String type, 
    									String var){
    	
    	return addGlobalVar(language, isPublic, false, isFinal, isStatic, isToInit, initValue, type, var);
    }
    
    
	public static String addGlobalVar(Language language, boolean isPublic, boolean isProtected,
			boolean isFinal, boolean isStatic, boolean isToInit,
			String initValue, String type, String var) {
		String code = "";
		if (language.equals(Language.JAVA)) {
			if (isPublic)
				code += "public";
			else if (isProtected)
				code += "protected";
			else
				code += "private";
			
			if (isFinal)
				code += " final";
			if (isStatic)
				code += " static";
			code += " " + getType(Language.JAVA, type);
			code += " " + var;
			if (isToInit) {
				code += " = ";
				if (initValue.equals(""))
					code += getJavaAttributesInitiator(type);
				else
					code += initValue;
			}
			code += ";";
		}
		if (language.equals(Language.CPP)) {
			if (isStatic)
				code += "static ";
			if (isFinal)
				code += " const";
			code += " " + getType(Language.CPP, type);
			code += " " + var;
			if (isFinal && isToInit) {
				code += " = ";
				if (initValue.equals(""))
					code += getCppAttributesInitiator(type);
				else
					code += initValue;
			}
			code += ";";
		}

		if (language.equals(Language.FLASH)) {
			if (isPublic)
				code += "public";
			else
				code += "private";
			if (isStatic)
				code += " static";
			code += " var " + var;
			code += ":" + getType(Language.FLASH, type);
			if (isToInit) {
				code += " = ";
				if (initValue.equals(""))
					code += "new " + getFlashAttributesInitiator(type);
				else
					code += initValue;
			}
			code += ";";
		}

		if (language.equals(Language.CSHARP)) {
			if (isPublic)
				code += "public";
			else
				code += "private";
			// if (isFinal) code += " sealed"; cannot be used for fields
			if (isStatic)
				code += " static";
			code += " " + getType(Language.CSHARP, type);
			code += " " + var;
			if (isToInit) {
				code += " = ";
				if (initValue.equals(""))
					code += getCsharpAttributesInitiator(type);
				else
					code += initValue;
			}
			code += ";";
		}

		return code;
	}

    // idem, en C++, pour les fonctions sont toutes publiques
	public static String writeFunctionDeclaration(Language language,
    									boolean isPublic, 
    									boolean isStatic,
    									boolean isSynchronized,
    									boolean isVoid,
    									String returnType, 
    									String functionName, 
    									String[][] params, 
    									boolean isThrownException,
    									String[] exceptions){
		
		return writeFunctionDeclaration(language, isPublic, false, !isPublic, isStatic, 
										isSynchronized, isVoid, returnType, functionName,
										params, isThrownException, exceptions);
	}
	
	 // idem, en C++, pour les fonctions sont toutes publiques
	public static String writeFunctionDeclaration(Language language,
    									boolean isPublic,
    									boolean isProtected,
    									boolean isPrivate,
    									boolean isStatic,
    									boolean isSynchronized,
    									boolean isVoid,
    									String returnType, 
    									String functionName, 
    									String[][] params, 
    									boolean isThrownException,
    									String[] exceptions){
		
		
		String code = "";
    	if (language.equals(Language.JAVA) ){
    		if (isPublic) code += "public";
    		else if (isProtected)  code += "protected";
    		else if (isPrivate)  code += "private";
    		
    		if (isStatic) code += " static";
    		if (isSynchronized) code += " synchronized";
    		if (isVoid) code += " void";
    		else code += " "+getType(Language.JAVA, returnType);
    		code += " "+ functionName + "(";
    		
    		if (params!=null && params.length > 0){
	    		for (int i = 0; i<params.length; i++){
	    			code += getJavaType(params[i][0]) + " " + params[i][1];
	    			if ((i+1) != params.length) code += ",";
	    		}
    		}
    		
    		code += ")";
    		if (isThrownException){
    			code+=" throws ";
    			for (int i=0; i < exceptions.length ; i++){
    				code+= exceptions[i];
    				if (i+1<exceptions.length) code+=", ";
    			}
    		}
    		code += " {";
    	}
    	if (language.equals(Language.CPP) ){
    		if (isStatic) code += "static ";
    		if (isVoid) code += "void ";
    		else code += " "+getType(Language.CPP, returnType);
    		code += " "+ functionName + "(";
    		
    		if (params!=null && params.length > 0){
	    		for (int i = 0; i<params.length; i++){
	    			code += getCppType(params[i][0]) + " " + params[i][1];
	    			if ((i+1) != params.length) code += ",";
	    		}
    		}
    		
    		code += ")"+"\n\t{";
    	}
    	if (language.equals(Language.FLASH) ){
    		if (isPublic) code += "public";
    		else code += "private";
    		if (isStatic) code += " static";
    		code += " function "+ functionName + "(";
    		
    		if (params!=null && params.length > 0){
	    		for (int i = 0; i<params.length; i++){
	    			code += params[i][1] + ":" + getFlashType(params[i][0]);
	    			if ((i+1) != params.length) code += ",";
	    		}
    		}
    		
    		code += ")";
    		if (!isVoid) code += ":"+getType(Language.FLASH, returnType);
    		code += " {";
    	}
    	if (language.equals(Language.CSHARP) ){
    		if (isPublic) code += "public";
    		else  code += "private";
    		if (isStatic) code += " static";
    		if (isSynchronized) code += " synchronized";
    		if (isVoid) code += " void";
    		else code += " "+getType(Language.CSHARP, returnType);
    		code += " "+ functionName + "(";
    		
    		if (params!=null && params.length > 0){
	    		for (int i = 0; i<params.length; i++){
	    			code += getCsharpType(params[i][0]) + " " + params[i][1];
	    			if ((i+1) != params.length) code += ",";
	    		}
    		}
    		
    		code += "){";
    	}
    	return code;
    }
    
    public static String stringLength(Language language, String s){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.CPP) ){
    		code = s+".length()";
    	}
    	if (language.equals(Language.FLASH) ){
    		code = "length("+s+")";
    	}
    	if (language.equals(Language.CSHARP) ){
    		code = s+".Length()";
    	}
    	
    	return code;
    }
  
    public static String add(Language language, 
    								boolean isToAdd,
    								String type,
    								String output, 
    								String[] itemsToAdd){
    	String code = "";
    	if (language.equals(Language.JAVA) || language.equals(Language.FLASH)|| language.equals(Language.CPP)|| language.equals(Language.CSHARP)){ 
    		code += output;
    		if (isToAdd) code += " += ";
    		else  code += " -= ";
    		if (itemsToAdd!=null) 
    			for (int i=0; i<itemsToAdd.length; i++){
	    			code += itemsToAdd[i];
	    			if ((i+1) != itemsToAdd.length) code += " + ";	
    			}
    		code += ";";
    	}
    	return code;
    }
    
    public static String addValueToValue(Language language, 
			boolean isToAdd,
			String[] itemsToAdd){
		String code = "";
		if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)   || language.equals(Language.CSHARP) ){ 
			code += "(";
			if (itemsToAdd!=null) 
				for (int i=0; i<itemsToAdd.length; i++){
					code += itemsToAdd[i];
					if ((i+1) != itemsToAdd.length) {
						if (isToAdd) code += " + ";
						else code += " - ";
					}
				}
			code += ")";
		}
		return code;
	}
    
    public static String increment(Language language, boolean isToIncrement,String valueToIncrement){
		String code = "";
		if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){ 
			if (isToIncrement) code += valueToIncrement+ "++;";
			else code += valueToIncrement+ "--;";
		}
		return code;
	}
    
    public static String addLocalVar(Language language, boolean isToInit, String initValue, boolean isASpecificConstructor, String specificConstructor, String type, String var){
    	String code = "";
		if (language.equals(Language.JAVA)){ 
			code += getType(Language.JAVA,type) + " " + var;
			if (isToInit) {
				code += " = ";
				if (isASpecificConstructor) code += "new "+specificConstructor;
				else{
					if (initValue.equals("")) code += getJavaAttributesInitiator(type);
					else code += initValue;
				}
			}
			code += ";";
		}	
		//TODO Initialiser une hashtable
		if (language.equals(Language.CPP) ){ 
			code += getType(Language.CPP,type) + " " + var;
			// Pas de valeur d'initialisation pour les objets crees
			if (isToInit && !getType(Language.CPP,type).equals("map")) {
				code +=  (initValue.equals("") && getCppAttributesInitiator(type).equals(""))?"":" = ";
				if (isASpecificConstructor) code += "new "+specificConstructor;
				if (initValue.equals("")) code += getCppAttributesInitiator(type);
				else code += initValue;
			}
			code += ";";
		}
		if (language.equals(Language.FLASH) ) {
			code += "var "+var+":"+getType(Language.FLASH,type);
			if (isToInit) {
				code += " = ";
				
				if (isASpecificConstructor) code += "new "+specificConstructor;
				else{
					if (initValue.equals("")) code += " new " + getFlashAttributesInitiator(type);
					else code += initValue;
				}
			}
			
			code += ";";
		}
		if (language.equals(Language.CSHARP) ){ 
			code += getType(Language.CSHARP,type) + " " + var;
			if (isToInit) {
				code += " = ";
				if (isASpecificConstructor) code += "new "+specificConstructor;
				else{
					if (initValue.equals("")) code += getCsharpAttributesInitiator(type);
					else code += initValue;
				}
			}
			code += ";";
		}	
		return code;
    }
    
    public static String addLocalVar(Language language, boolean isToInit, String initValue, String type, String var){
    	return addLocalVar(language,isToInit,initValue,false,"",type,var);
	}
    
    
    public static String testEqualType(Language language, boolean isEqual, String type, String[] itemsToEquals){
    	String code = "";
    	if (language.equals(Language.JAVA) ){ 
    		String lType = getPrimitiveJavaType(type);
    		if (lType.equals("null") // for no primitive types
    				|| lType.equals("String")){
    			if (!isEqual) code += "!";
    			code += itemsToEquals[0];
    			code += ".equals(" + itemsToEquals[1] + ")";
    		}
    		else {
    			code += itemsToEquals[0];
    			if (isEqual) code += " == ";
    			else code += " != ";
    			code += itemsToEquals[1];
    		}
    	}
    	if (language.equals(Language.CPP) ){ 
    		String lType = getPrimitiveCppType(type);
    		
    		 // for no primitive types
    		if (lType.equals("null")){
    			if (!isEqual) code += "!";
    			code += itemsToEquals[0];
    			code += ".equals(" + itemsToEquals[1] + ")";
    		}
    		// pas de methode equals, on fait compare() == 0
    	    else if (lType.equals("std::string")){
    	    	if (!isEqual) code += "!";
    	    	code +="(("+itemsToEquals[0]+".compare("+itemsToEquals[1]+")) == 0)";	
    	    }
    		else {
    			code += itemsToEquals[0];
    			if (isEqual) code += " == ";
    			else code += " != ";
    			code += itemsToEquals[1];
    		}
    	}
    	if (language.equals(Language.FLASH) ){ 
    		if (type.equals("char")) code += replaceCharQuote(language,itemsToEquals[0]);
    		else code += itemsToEquals[0];
    		if (isEqual) code += "===";
    		else code += "!==";
    		if (type.equals("char")) code += replaceCharQuote(language,itemsToEquals[1]);
    		else code += itemsToEquals[1];
    	}
    	if (language.equals(Language.CSHARP) ){ 
    		String lType = getPrimitiveCsharpType(type);
    		if (lType.equals("null") // for no primitive types
    				|| lType.equals("String")){
    			if (!isEqual) code += "!";
    			code += itemsToEquals[0];
    			code += ".Equals(" + itemsToEquals[1] + ")";
    		}
    		else {
    			code += itemsToEquals[0];
    			if (isEqual) code += " == ";
    			else code += " != ";
    			code += itemsToEquals[1];
    		}
    	}
    	return code;
    }
    
    // inutile en C++
    private static String replaceCharQuote(Language language, String string) {
    	String stringParsed = "";
    	if (language.equals(Language.FLASH) ){
			//no ' for char but "
			
			for(int i = 0; i < string.length(); i++){
				if (string.charAt(i) == '\'')  stringParsed += '\"';
				else  stringParsed += string.charAt(i);
			}
		}
		return stringParsed;
	}

	public static String testSup(Language language, boolean isSup, String[] itemsToTest){
    	String code = "";
    	if (language.equals(Language.JAVA)  ||language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){ 
    		code += itemsToTest[0];
    		if (isSup) code += ">";
    		else code += "<";
    		code += itemsToTest[1];
    	}
    	return code;
    }
    
    public static String testAnd(Language language, boolean isToAnd, String[] itemsToTest){
    	String code = "";
    	if (language.equals(Language.JAVA)  ||language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){ 
    		code += itemsToTest[0];
    		if (isToAnd) code += " && ";
    		else code += " || ";
    		code += itemsToTest[1];
    	}
    	return code;
    }
    
    public static String ifDeclaration(Language language, String _condition){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){ 
    		code += "if (" +_condition+ ") {";;
    	}
    	return code;
    }
    
    public static String closeBlock(Language language){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){ 
    		code += "}" ;
    	}
    	return code;
    }
    
    public static String endOfLineMarker(Language language){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){ 
    		code += ";" ;
    	}
    	return code;
    }
    
    public static String elseDeclaration(Language language, String _else){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){ 
    		code += "else {\n" +_else;
    	}
    	return code;
    }
    
    public static String elseIfDeclaration(Language language, String _condition){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){ 
    		code += "else " + ifDeclaration(language, _condition);
    	}
    	return code;
    }
    
    public static String returnValue(Language language, String valueToReturn){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){ 
    		code += "return " + valueToReturn+";";
    	}
    	return code;
    }
    
    //TODO
    public static String arrayDeclaration(Language language, String type, int dimension){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.CSHARP) ){ 
    		code += type;
    		for(int i = 0; i<dimension; i++){
    			code += "[]";
    		}
    	}// TODO: dimension
    	// if type isn't a primitive one, with have to put a pointer
    	if(language.equals(Language.CPP) ){
    		code+= "std::vector<"+type+">";
    	}
    	if (language.equals(Language.FLASH) ) code += "Array";
    	return code;	
    }
    
    public static String arrayLength(Language language, String array){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH) ){ 
    		code += array+".length";
    	}
    	if(language.equals(Language.CPP) ){
    		code+= array+".size()";
    	}
    	if(language.equals(Language.CSHARP) ){
    		code+= array+".Length";
    	}
    	return code;	
    }
    
    
    public static String arrayAccess(Language language, String tableName, String accessTo){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){  
    		code+= tableName + "["+accessTo+"]";
    	}
        return code;
    }
    
    public static String arrayAccess(Language language, String accessTo){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){  
    		code+="["+accessTo+"]";
    	}
        return code;
    }
    
    // TODO
    public static String arrayInit(Language language, String type, int dimension, String nb){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.CSHARP) ){ 
    		code += type;
    		code += "["+nb+"]";
    		for(int i = 0; i<dimension-1; i++){
    			 code += "[]";
    		}
    	}
    	if(language.equals(Language.CPP) ){
    		code+= ".resize("+nb+")";
    	}
    	if (language.equals(Language.FLASH) ){
    		code += "new Array()";
    	}
    	return code;	
    }
    
    public static String forLoopDeclaration(Language language, String var, String min, String max){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){ 
    		code += "for(int "+ var +"="+ min +"; "+ var+" <"+ max +"; "+var+"++){";
    	}
    	if (language.equals(Language.FLASH) ){
    		code += "for(var "+ var +"="+ min +"; "+ var+" <"+ max +"; "+var+"++){";
    	}
    	return code;
    }
    
    public static String loopInterrupt(Language language){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ) code += "break;";
    	return code;
    	
    }
    
    public static String testInstanceOfType(Language language, String instance, String type){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH) )
    		code += instance+" instanceof "+ type;
    	//(typeid(instance) == typeid(type))
    	if (language.equals(Language.CPP) )
    		code += "(typeid("+instance+") == typeid("+type+"))";
    	if (language.equals(Language.CSHARP) )
    		code += instance+" is "+ type;
    	return code;
    }
    
    
    public static String affectValueToVar(Language language, String var, String value){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){
    		code += var + " = "+ value +";";
    	}
    	//if(language.equals(Language.CPP) ){
    		//code += ((value.equals(""))?var+" AAA ;": var + " = BBBB"+ value +"BBBB;");
    	//}
    		return code;
    }
    
    
    public static String callMethod(Language language, 
    									boolean isAnObjectMethod,
    									String object,
    									String name,
    									String[] params){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){
    		if (isAnObjectMethod) code += object + ".";
    		code += name + "(";
    		if (params != null)
	    		for (int i = 0; i<params.length; i++){
	    			code += params[i];
	    			if (i+1!=params.length) code += ",";
	    		}
    		code+=")";
    	}
    	return code;
    }
    
    // TODO
    public static String cast(Language language, String type, String var){
    	String code = "";
    	if (language.equals(Language.JAVA) )
    		code += "(("+getJavaObjectType(type)+") "+ var+")";
    	if (language.equals(Language.FLASH) )
    		code += getFlashType(type)+"("+ var +")";
    	if (language.equals(Language.CSHARP) )
    		code += "(("+getCsharpObjectType(type)+") "+ var+")";
    	return code;
    }
    
    public static String castPrimitive(Language language, String type, String var){
    	String code = "";
    	if (language.equals(Language.JAVA) )
    		code += "("+getJavaType(type)+") "+ var;
    	if (language.equals(Language.CPP) )
    		code += "("+getCppType(type)+") "+ var;
    	if (language.equals(Language.FLASH) )
    		code += getFlashType(type)+"("+ var +")";
    	if (language.equals(Language.CSHARP) )
    		code += "("+getCsharpType(type)+") "+ var;
    	return code;
    }
    
    //TODO
    public static String parseStringType(Language language, String type, String stringToParse){
    	String code = "";
    	if (language.equals(Language.JAVA) ) 
    		code += getJavaTypeStringDecoder(getJavaType(type),stringToParse);
    	if (language.equals(Language.FLASH) ) 
    		code += getFlashTypeDecoder(getFlashType(type),stringToParse);
    	if (language.equals(Language.CSHARP) ) 
    		code += getCsharpTypeStringDecoder(getCsharpType(type),stringToParse);
   
    	return code;
    }

    public static String whileLoopDeclaration(Language language, String condition){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)   || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ) 
    		code += "while ("+ condition + ") {";
    	return code;
    }
    
    public static String switchDeclaration(Language language, String element){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)   || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ) 
    		code += "switch ("+ element + ") {";
    	return code;
    }
    
    public static String caseDeclaration(Language language, String element){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)   || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ) 
    		code += "case "+ element + " :";
    	return code;
    }
    
    public static String readBinaryType(Language language, String type, String binaryVar){
    	String code = "";
    	if (language.equals(Language.JAVA) ) 
    		code += getJavaTypeDecoder(type,binaryVar);
    	if (language.equals(Language.CPP) ) 
    		code += getCppTypeDecoder(type,binaryVar);
    	if (language.equals(Language.CSHARP) ) 
    		code += getCsharpTypeDecoder(type,binaryVar);
    	return code;
    }
    
    
    public static String writeBinaryType(Language language, String type, String valueToWrite, String binaryVar){
    	String code = "";
    	if (language.equals(Language.JAVA) ) 
    		code += getJavaTypeEncoder(type, valueToWrite, binaryVar);
    	if (language.equals(Language.CPP) ) 
    		code += getCppTypeEncoder(type, valueToWrite, binaryVar);
    	if (language.equals(Language.CSHARP) ) 
    		code += getCsharpTypeEncoder(type, valueToWrite, binaryVar);
    	return code;
    }
    
    
    private static String getJavaTypeDecoder(String type, String binaryVar){
    	String code = "";
    	String javaType = getJavaType(type);
    	if (javaType.equals("long")) code += binaryVar+".readLong()";
    	if (javaType.equals("int")) code += binaryVar+".readInt()";
    	if (javaType.equals("short")) code += binaryVar+".readShort()";
    	if (javaType.equals("float")) code += binaryVar+".readFloat()";
    	if (javaType.equals("double")) code += binaryVar+".readDouble()";
    	if (javaType.equals("byte")) code += binaryVar+".readByte()";
    	if (javaType.equals("boolean")) code += binaryVar+".readBoolean()";
    	if (javaType.equals("String")) code += binaryVar+".readUTF()";
    	if (javaType.equals("char")) code += binaryVar+".readChar()";
    	return code;
    }

    
    private static String getJavaTypeEncoder(String type, String valueToWrite, String binaryVar){
    	String code = "";
    	String javaType = getJavaType(type);
    	if (javaType.equals("long")) code += binaryVar+".writeLong("+valueToWrite+");";
    	if (javaType.equals("int")) code +=  binaryVar+".writeInt("+valueToWrite+");";
    	if (javaType.equals("short")) code += binaryVar+".writeShort("+valueToWrite+");";
    	if (javaType.equals("float")) code += binaryVar+".writeFloat("+valueToWrite+");";
    	if (javaType.equals("double")) code += binaryVar+".writeDouble("+valueToWrite+");";
    	if (javaType.equals("byte")) code += binaryVar+".writeByte("+valueToWrite+");";
    	if (javaType.equals("boolean")) code += binaryVar+".writeBoolean("+valueToWrite+");";
    	if (javaType.equals("String")) code += binaryVar+".writeUTF("+valueToWrite+");";
    	if (javaType.equals("char")) code += binaryVar+".writeChar("+valueToWrite+");";
    	return code;
    }
    
    //TODO type char
    private static String getCppTypeDecoder(String type, String binaryVar){
    	String code = "";
    	String cppType = getCppType(type);
    	if (cppType.equals("long long")) code += binaryVar+".readLong()";
    	if (cppType.equals("int")) code += binaryVar+".readInt()";
    	if (cppType.equals("short")) code += binaryVar+".readShort()";
    	if (cppType.equals("float")) code += binaryVar+".readFloat()";
    	if (cppType.equals("double")) code += binaryVar+".readDouble()";
    	if (cppType.equals("char")) code += binaryVar+".readByte()";
    	if (cppType.equals("bool")) code += binaryVar+".readBool()";
    	if (cppType.equals("std::string")) code += binaryVar+".readUTF()";
    	//if (cppType.equals("char")) code += binaryVar+".readChar()";
    	return code;
    }
    
   
    
    //TODO type char
    private static String getCppTypeEncoder(String type, String valueToWrite, String binaryVar){
    	String code = "";
    	String cppType = getCppType(type);
    	if (cppType.equals("long long")) code += binaryVar+".writeLong("+valueToWrite+");";
    	if (cppType.equals("int")) code +=  binaryVar+".writeInt("+valueToWrite+");";
    	if (cppType.equals("short")) code += binaryVar+".writeShort("+valueToWrite+");";
    	if (cppType.equals("float")) code += binaryVar+".writeFloat("+valueToWrite+");";
    	if (cppType.equals("double")) code += binaryVar+".writeDouble("+valueToWrite+");";
    	if (cppType.equals("char")) code += binaryVar+".writeByte("+valueToWrite+");";
    	if (cppType.equals("bool")) code += binaryVar+".writeBool("+valueToWrite+");";
    	if (cppType.equals("std::string")) code += binaryVar+".writeUTF("+valueToWrite+");";
    	//if (cppType.equals("char")) code += binaryVar+".writeChar("+valueToWrite+");";
    	return code;
    }
    
    
    
    private static String getCsharpTypeDecoder(String type, String binaryVar){
    	String code = "";
    	String javaType = getCsharpType(type);
    	if (javaType.equals("long")) code += binaryVar+".ReadInt64()";
    	if (javaType.equals("int")) code += binaryVar+".ReadInt32()";
    	if (javaType.equals("short")) code += binaryVar+".ReadInt16()";
    	if (javaType.equals("float")) code += binaryVar+".ReadSingle()";
    	if (javaType.equals("double")) code += binaryVar+".ReadDouble()";
    	if (javaType.equals("byte")) code += binaryVar+".ReadByte()";
    	if (javaType.equals("bool")) code += binaryVar+".ReadBoolean()";
    	if (javaType.equals("String")) code += binaryVar+".ReadUTF()";
    	if (javaType.equals("char")) code += binaryVar+".ReadChar()";
    	return code;
    }
    
    private static String getCsharpTypeEncoder(String type, String valueToWrite, String binaryVar){
    	String code = "";
    	String javaType = getCsharpType(type);
    	if (javaType.equals("long")) code += binaryVar+".WriteInt64("+valueToWrite+");";
    	if (javaType.equals("int")) code +=  binaryVar+".WriteInt32("+valueToWrite+");";
    	if (javaType.equals("short")) code += binaryVar+".WriteInt16("+valueToWrite+");";
    	if (javaType.equals("float")) code += binaryVar+".WriteSingle("+valueToWrite+");";
    	if (javaType.equals("double")) code += binaryVar+".WriteDouble("+valueToWrite+");";
    	if (javaType.equals("byte")) code += binaryVar+".WriteByte("+valueToWrite+");";
    	if (javaType.equals("boolean")) code += binaryVar+".WriteBoolean("+valueToWrite+");";
    	if (javaType.equals("String")) code += binaryVar+".WriteUTF("+valueToWrite+");";
    	if (javaType.equals("char")) code += binaryVar+".WriteChar("+valueToWrite+");";
    	return code;
    }
    
        
    //TODO pas besoin en C++?
    public static String getObjectTypeValue(Language language, String type, String string) {
    	if (language.equals(Language.JAVA) )
	    	return getJavaObjectTypeValue(type,string);
    	if (language.equals(Language.CSHARP) )
	    	return getCsharpObjectTypeValue(type,string);
    	
    	return "";
	}

	private static String getJavaObjectTypeValue(String type, String s) {
    	String code = "";
    	String javaType = getJavaType(type);
    	if (javaType.equals("String")) code+= s;
    	if (javaType.equals("long")) code += "("+s+")"+".longValue()";
    	if (javaType.equals("int")) code += "("+s+")"+".intValue()";
    	if (javaType.equals("short")) code += "("+s+")"+".shortValue()";
    	if (javaType.equals("float")) code += "("+s+")"+".floatValue()";
    	if (javaType.equals("double")) code += "("+s+")"+".doubleValue()";
    	if (javaType.equals("byte")) code += "("+s+")"+".byteValue()";
    	if (javaType.equals("boolean")) code += "("+s+")"+".booleanValue()";
    	if (javaType.equals("char")) code += "("+s+")"+".charValue()";
    	return code;
	}
	
	private static String getCsharpObjectTypeValue(String type, String s) {
    	String code = "";
    	String javaType = getCsharpType(type);
    	if (javaType.equals("String")) code+= s;
    	if (javaType.equals("long")) code += "Int64.Parse("+s+")";
    	if (javaType.equals("int")) code += "Int32.Parse("+s+")";
    	if (javaType.equals("short")) code += "Int16.Parse("+s+")";
    	if (javaType.equals("float")) code += "Float.Parse("+s+")";
    	if (javaType.equals("double")) code += "Double.Parse("+s+")";
    	if (javaType.equals("byte")) code += "Byte.Parse("+s+")";
    	if (javaType.equals("boolean")) code += "Boolean.Parse("+s+")";
    	if (javaType.equals("char")) code += "Char.Parse("+s+")";
    	return code;
	}

    public static String getType(Language language, String type){
    	if (language.equals(Language.JAVA))
	    	return getJavaType(type);
    	if (language.equals(Language.CPP))
    		return getCppType(type);
    	if (language.equals(Language.FLASH))
    		return getFlashType(type);
    	if (language.equals(Language.CSHARP))
    		return getCsharpType(type);
    	
    
    	return "";
    }

    public static String newType(Language language, String type, String init){
    	return newType(language,type,new String[]{init});
    }
   
    public static String newType(Language language, String type, String[] init){
    	String code = "";
    	if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){
    		code += "new ";
    		code += getObjectType(language,type);
  
    		code += "(";
    		for(int i=0; i<init.length; i++){
    			code += init[i];
    			if (i != init.length - 1) code += ",";
    		}
    		code += ")";
    	}
    	return code;
    }
   
    private static String getObjectType(Language language, String type) {
		if (language.equals(Language.JAVA) ) return getJavaObjectType(type);
		if (language.equals(Language.CPP) ) return getCppType(type);
		if (language.equals(Language.FLASH) ) return getFlashType(type);
		if (language.equals(Language.CSHARP) ) return getCsharpType(type);
		return null;
	}


	/**
     * Return the type of elementType.
     * 
     * @param elementType
     * @return id 
     */
    private static String getJavaObjectType(String elementType) {
       
    	if (elementType.toLowerCase().equals("xsd:string") ||
        		elementType.toLowerCase().equals("string")) 
            return "String";
        if (elementType.toLowerCase().equals("xsd:char")||
        		elementType.toLowerCase().equals("char")) 
            return "Character";
        if (elementType.toLowerCase().equals("xsd:int")||
        		elementType.toLowerCase().equals("int")) 
            return "Integer";
        if (elementType.toLowerCase().equals("xsd:long")||
        		elementType.toLowerCase().equals("long")) 
            return "Long";
        if (elementType.toLowerCase().equals("xsd:double")||
        		elementType.toLowerCase().equals("double")) 
            return "Double";
        if (elementType.toLowerCase().equals("xsd:float")||
        		elementType.toLowerCase().equals("float")) 
            return "Float";
        if (elementType.toLowerCase().equals("xsd:short")||
        		elementType.toLowerCase().equals("short"))
            return "Short";
        if (elementType.toLowerCase().equals("xsd:byte")||
        		elementType.toLowerCase().equals("byte")) 
            return "Byte";
        if (elementType.toLowerCase().equals("xsd:boolean")||
        		elementType.toLowerCase().equals("boolean"))
            return "Boolean";
        else return elementType;
    } 
    
  
    // flash types String, Boolean, Number
    public static String getFlashType(String type){
    	String typeLowCase = type.toLowerCase();
    	if (typeLowCase.equals("xsd:string") 
    			|| typeLowCase.equals("string")
    		    || typeLowCase.equals("xsd:char")
    		    || typeLowCase.equals("char")) 
    		return "String";
    	else if (typeLowCase.equals("xsd:boolean")
            	|| typeLowCase.equals("boolean"))
        	return "Boolean";
    	else if (typeLowCase.equals("xsd:int")
        		|| typeLowCase.equals("int")
        	|| typeLowCase.equals("xsd:short")
        	    || typeLowCase.equals("short")
        	|| typeLowCase.equals("xsd:long")
        		|| typeLowCase.equals("long")
        	|| typeLowCase.equals("xsd:byte")
        		|| typeLowCase.equals("byte")
        	|| typeLowCase.equals("xsd:double")
        	|| typeLowCase.equals("double"))
        	return "Number";
        return type;
    }
    
    
    /**
     * Return the type of elementType.
     * 
     * @param elementType
     * @return id 
     */
    private static String getCsharpObjectType(String elementType) {
       
    	if (elementType.toLowerCase().equals("xsd:string") ||
        		elementType.toLowerCase().equals("string")) 
            return "String";
        if (elementType.toLowerCase().equals("xsd:char")||
        		elementType.toLowerCase().equals("char")) 
            return "Char";
        if (elementType.toLowerCase().equals("xsd:int")||
        		elementType.toLowerCase().equals("int")) 
            return "Int32";
        if (elementType.toLowerCase().equals("xsd:long")||
        		elementType.toLowerCase().equals("long")) 
            return "Int64";
        if (elementType.toLowerCase().equals("xsd:double")||
        		elementType.toLowerCase().equals("double")) 
            return "Double";
        if (elementType.toLowerCase().equals("xsd:short")||
        		elementType.toLowerCase().equals("short"))
            return "Int16";
        if (elementType.toLowerCase().equals("xsd:byte")||
        		elementType.toLowerCase().equals("byte")) 
            return "Byte";
        if (elementType.toLowerCase().equals("xsd:boolean")||
        		elementType.toLowerCase().equals("boolean"))
            return "Boolean";
        else return elementType;
    } 

    // java types String, boolean, long, int, short, double, byte
    public static String getJavaType(String type){
    	String lType = getPrimitiveJavaType(type);

    	if (lType.equals("null"))
    		return type;
    	else return lType;
    }
    
    // java types String, boolean, long, int, short, double, byte
    public static String getCsharpType(String type){
    	String lType = getPrimitiveCsharpType(type);

    	if (lType.equals("null"))
    		return type;
    	else return lType;
    }
    
   
    // java types String, boolean, long, int, short, double, byte
    public static String getPrimitiveJavaType(String type){
    	String typeLowCase = type.toLowerCase();
    	if (typeLowCase.equals("xsd:string") 
    			|| typeLowCase.equals("string")) 
    		return "String";
    	if (typeLowCase.equals("xsd:char") 
    			|| typeLowCase.equals("char")) 
    		return "char";
    	else if (typeLowCase.equals("xsd:boolean")
            	|| typeLowCase.equals("boolean"))
        	return "boolean";
    	else if (typeLowCase.equals("xsd:int")
        		|| typeLowCase.equals("int"))
        	return "int";
    	else if (typeLowCase.equals("xsd:short")
        	    || typeLowCase.equals("short"))
        	return "short";
    	else if (typeLowCase.equals("xsd:long")
        		|| typeLowCase.equals("long"))
        	return "long";
        else if (typeLowCase.equals("xsd:byte")
        		|| typeLowCase.equals("byte"))
        	return "byte";
        else if (typeLowCase.equals("xsd:float")
            	|| typeLowCase.equals("float"))
            	return "float";
        else if (typeLowCase.equals("xsd:double")
        	|| typeLowCase.equals("double"))
        	return "double";
        else return "null";
    }
    
    // java types String, boolean, long, int, short, double, byte
    public static String getPrimitiveCsharpType(String type){
    	String typeLowCase = type.toLowerCase();
    	if (typeLowCase.equals("xsd:string") 
    			|| typeLowCase.equals("string")) 
    		return "String";
    	if (typeLowCase.equals("xsd:char") 
    			|| typeLowCase.equals("char")) 
    		return "char";
    	else if (typeLowCase.equals("xsd:boolean")
            	|| typeLowCase.equals("boolean"))
        	return "bool";
    	else if (typeLowCase.equals("xsd:int")
        		|| typeLowCase.equals("int"))
        	return "int";
    	else if (typeLowCase.equals("xsd:short")
        	    || typeLowCase.equals("short"))
        	return "short";
    	else if (typeLowCase.equals("xsd:long")
        		|| typeLowCase.equals("long"))
        	return "long";
        else if (typeLowCase.equals("xsd:byte")
        		|| typeLowCase.equals("byte"))
        	return "byte";
        else if (typeLowCase.equals("xsd:float")
            	|| typeLowCase.equals("float"))
            	return "float";
        else if (typeLowCase.equals("xsd:double")
        	|| typeLowCase.equals("double"))
        	return "double";
        else return "null";
    }
    
    public static String getCppType(String type){
    	String lType = getPrimitiveCppType(type);
    	if (lType.equals("null"))
    		return type;
    	else return lType;
    }
    
    public static boolean isPrimitiveCppType(String type){
    	String lType = getPrimitiveCppType(type);
    	if (lType.equals("null"))
    		return false;
    	else return true;
    }
    
    public static String getPrimitiveCppType(String type){
    	String typeLowCase = type.toLowerCase();
    	if (typeLowCase.equals("xsd:string") 
    			|| typeLowCase.equals("string")
    			|| typeLowCase.equals("std::string")) 
    		return "std::string";
    	if (typeLowCase.equals("xsd:char") 
    			|| typeLowCase.equals("char")) 
    		return "char";
    	else if (typeLowCase.equals("xsd:boolean")
            	|| typeLowCase.equals("boolean")
            	|| typeLowCase.equals("bool"))
        	return "bool";
    	else if (typeLowCase.equals("xsd:int")
        		|| typeLowCase.equals("int"))
        	return "int";
    	else if (typeLowCase.equals("xsd:short")
        	    || typeLowCase.equals("short"))
        	return "short";
    	else if (typeLowCase.equals("xsd:long")
        		|| typeLowCase.equals("long")
    			|| typeLowCase.equals("long long"))
        	return "long long";
        else if (typeLowCase.equals("xsd:byte")
        		|| typeLowCase.equals("byte")) 
        	return "char";
        else if (typeLowCase.equals("xsd:float")
        	|| typeLowCase.equals("float"))
        	return "float";
        else if (typeLowCase.equals("xsd:double")
        	|| typeLowCase.equals("double"))
        	return "double";
        else if (type.equals("Hashtable"))
            	return "map";
        else return "null";
    }
    
    private static String getJavaTypeStringDecoder(String type, String stringToParse){
    	String code = "";
    	String javaType = getJavaType(type);
    	if (javaType.equals("String")) code+= stringToParse;
    	if (javaType.equals("long")) code += "Long.parseLong("+stringToParse+")";
    	if (javaType.equals("int")) code += "Integer.parseInt("+stringToParse+")";
    	if (javaType.equals("short")) code += "Short.parseShort("+stringToParse+")";
    	if (javaType.equals("float")) code += "Float.parseFloat("+stringToParse+")";
    	if (javaType.equals("double")) code += "Double.parseDouble("+stringToParse+")";
    	if (javaType.equals("byte")) code += "Byte.parseByte("+stringToParse+")";
    	if (javaType.equals("char")) code += stringToParse+".charAt(0)";
    	// special handling of booleans to decode them from int value 0 or 1
    	// instead of Boolean class' method decoding it from a string "true" or "false"
    	if (javaType.equals("boolean")){
    		code += "decodeBoolean("+stringToParse+")";
    		Generator.isThereABooleanAttribute = true;
    	}
    	return code;

    }

    private static String getCsharpTypeStringDecoder(String type, String stringToParse){
    	String code = "";
    	String csharpType = getCsharpType(type);
    	if (csharpType.equals("String")) code+= stringToParse;
    	if (csharpType.equals("long")) code += "Int64.Parse("+stringToParse+")";
    	if (csharpType.equals("int")) code += "Int32.Parse("+stringToParse+")";
    	if (csharpType.equals("short")) code += "Int16.Parse("+stringToParse+")";
    	if (csharpType.equals("float")) code += "Float.Parse("+stringToParse+")";
    	if (csharpType.equals("double")) code += "Double.Parse("+stringToParse+")";
    	if (csharpType.equals("byte")) code += "Byte.Parse("+stringToParse+")";
    	if (csharpType.equals("char")) code += "Char.Parse("+stringToParse+")";
    	// special handling of booleans to decode them from int value 0 or 1
    	// instead of Boolean class' method decoding it from a string "true" or "false"
    	if (csharpType.equals("bool")){
    		code += "decodeBoolean("+stringToParse+")";
    		Generator.isThereABooleanAttribute = true;
    	}
    	return code;

    }
    
    // specific flash methods
    private static String getFlashTypeDecoder(String type, String s){
    	if (type.equals("Number")) return "parseInt("+s+")";
    	if (type.equals("Boolean")) return s;
    	return s;
    }
    
    public static String getAttributesInitiator(Language language, String type){
    	String code = "";
    	if (language.equals(Language.JAVA) ) code += getJavaAttributesInitiator(type);
    	if (language.equals(Language.CPP) ) code += getCppAttributesInitiator(type);
    	if (language.equals(Language.FLASH) ) code += getFlashAttributesInitiator(type);
    	if (language.equals(Language.CSHARP) ) code += getCsharpAttributesInitiator(type);
    	return code;
    }
    
    // flash types String, Boolean, Number
    private static String getJavaAttributesInitiator(String typeJava){
    	String typeLowCase = typeJava.toLowerCase();
    	if (typeLowCase.equals("string")) 
    		return "\"\"";
    	else if (typeLowCase.equals("char"))
        	return "\' \'";
    	else if (typeLowCase.equals("boolean"))
        	return "false";
    	else if (typeLowCase.equals("int")
        	    || typeLowCase.equals("short")
        		|| typeLowCase.equals("long")
        		|| typeLowCase.equals("byte")
        		|| typeLowCase.equals("float")
        		|| typeLowCase.equals("double"))
        	return "0";
    	else if (containsTableMarker(Language.JAVA,typeJava)) return "new " + typeJava;
    	else return "new "+typeJava+"()";
    }
    
    private static String getFlashAttributesInitiator(String typeFlash){
    	String typeLowCase = typeFlash.toLowerCase();
    	if (typeLowCase.equals("string")||typeLowCase.equals("char")) return "String(\"\")";
    	else if (typeLowCase.equals("boolean")) return "Boolean(false)";
    	else if (typeLowCase.equals("number")) return "Number(0)";
    	else if (containsTableMarker(Language.FLASH,typeFlash)) return typeFlash;
    	return typeFlash+"()";
    }
    
    private static String getCsharpAttributesInitiator(String type){
    	String typeLowCase = type.toLowerCase();
    	if (typeLowCase.equals("string")) 
    		return "\"\"";
    	else if (typeLowCase.equals("char"))
        	return "\' \'";
    	else if (typeLowCase.equals("bool"))
        	return "false";
    	else if (typeLowCase.equals("int")
        	    || typeLowCase.equals("short")
        		|| typeLowCase.equals("long")
        		|| typeLowCase.equals("byte")
        		|| typeLowCase.equals("float")
        		|| typeLowCase.equals("double"))
        	return "0";
    	else if (containsTableMarker(Language.CSHARP,type)) return "new " + type;
    	else return "new "+type+"()";
    }
    
    private static String getCppAttributesInitiator(String typeCpp){
    	String typeLowCase = typeCpp.toLowerCase();
    	if (typeLowCase.equals("std::string")) 
    		return "\"\"";
    	else if (typeLowCase.equals("bool"))
        	return "false";
    	else if (/*typeLowCase.equals("char")
    			||*/typeLowCase.equals("int")
    			||typeLowCase.equals("char")
        	    || typeLowCase.equals("short")
        		|| typeLowCase.equals("long long")
        		|| typeLowCase.equals("byte")
        		|| typeLowCase.equals("float")
        		|| typeLowCase.equals("double"))
        	return "0";
    	// ne pas initialiser (ex: Player p;)
    	else return "";
    }
    

    private static boolean containsTableMarker(Language language, String type) {
    	if (language.equals(Language.JAVA) ){ 
	        if (type.indexOf('[') != -1) return true;
			else return false;
    	}
    	if (language.equals(Language.CPP) ){ 
	        if (type.indexOf('[') != -1) return true;
			else return false;
    	}
    	if (language.equals(Language.FLASH) ){
    		if (type.indexOf("Array") != -1) return true;
    		else return false;
    	}
    	if (language.equals(Language.CSHARP) ){ 
	        if (type.indexOf('[') != -1) return true;
	    	else return false;
    	}
    	return false;
	}


	public static String getLanguageFileExtension(Language language) {
		String extension = "";
		if (language.equals(Language.JAVA)) 
			extension = "java";
		if (language.equals(Language.FLASH))
			extension = "as";
		if (language.equals(Language.CPP)) 
			extension = "h";
		if (language.equals(Language.CSHARP)) 
			extension = "cs";
		return extension;
	}
}
